@@ -1,5 +1,5 @@
 /* Transformations based on profile information for values.
-   Copyright (C) 2003-2015 Free Software Foundation, Inc.
+   Copyright (C) 2003-2017 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -20,71 +20,27 @@ along with GCC; see the file COPYING3.  If not see
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
-#include "tm.h"
-#include "hash-set.h"
-#include "machmode.h"
-#include "vec.h"
-#include "double-int.h"
-#include "input.h"
-#include "alias.h"
-#include "symtab.h"
-#include "wide-int.h"
-#include "inchash.h"
+#include "backend.h"
+#include "rtl.h"
 #include "tree.h"
+#include "gimple.h"
+#include "cfghooks.h"
+#include "ssa.h"
+#include "cgraph.h"
+#include "coverage.h"
+#include "data-streamer.h"
+#include "diagnostic.h"
 #include "fold-const.h"
 #include "tree-nested.h"
 #include "calls.h"
-#include "rtl.h"
-#include "hashtab.h"
-#include "hard-reg-set.h"
-#include "function.h"
-#include "flags.h"
-#include "statistics.h"
-#include "real.h"
-#include "fixed-value.h"
-#include "insn-config.h"
-#include "expmed.h"
-#include "dojump.h"
-#include "explow.h"
-#include "emit-rtl.h"
-#include "varasm.h"
-#include "stmt.h"
 #include "expr.h"
-#include "predict.h"
-#include "dominance.h"
-#include "cfg.h"
-#include "basic-block.h"
 #include "value-prof.h"
-#include "recog.h"
-#include "insn-codes.h"
-#include "optabs.h"
-#include "regs.h"
-#include "tree-ssa-alias.h"
-#include "internal-fn.h"
 #include "tree-eh.h"
-#include "gimple-expr.h"
-#include "is-a.h"
-#include "gimple.h"
 #include "gimplify.h"
 #include "gimple-iterator.h"
-#include "gimple-ssa.h"
 #include "tree-cfg.h"
-#include "tree-phinodes.h"
-#include "ssa-iterators.h"
-#include "stringpool.h"
-#include "tree-ssanames.h"
-#include "diagnostic.h"
 #include "gimple-pretty-print.h"
-#include "coverage.h"
-#include "gcov-io.h"
-#include "timevar.h"
 #include "dumpfile.h"
-#include "profile.h"
-#include "hash-map.h"
-#include "plugin-api.h"
-#include "ipa-ref.h"
-#include "cgraph.h"
-#include "data-streamer.h"
 #include "builtins.h"
 #include "params.h"
 #include "tree-chkp.h"
@@ -139,18 +95,12 @@ along with GCC; see the file COPYING3.  If not see
 
    Limitations / FIXME / TODO:
    * Only one histogram of each type can be associated with a statement.
-   * Currently, HIST_TYPE_CONST_DELTA is not implemented.
-     (This type of histogram was originally used to implement a form of
-     stride profiling based speculative prefetching to improve SPEC2000
-     scores for memory-bound benchmarks, mcf and equake.  However, this
-     was an RTL value-profiling transformation, and those have all been
-     removed.)
    * Some value profile transformations are done in builtins.c (?!)
    * Updating of histograms needs some TLC.
    * The value profiling code could be used to record analysis results
      from non-profiling (e.g. VRP).
    * Adding new profilers should be simplified, starting with a cleanup
-     of what-happens-where andwith making gimple_find_values_to_profile
+     of what-happens-where and with making gimple_find_values_to_profile
      and gimple_value_profile_transformations table-driven, perhaps...
 */
 
@@ -169,7 +119,7 @@ static bool gimple_ic_transform (gimple_stmt_iterator *);
 
 histogram_value
 gimple_alloc_histogram_value (struct function *fun ATTRIBUTE_UNUSED,
-			      enum hist_type type, gimple stmt, tree value)
+			      enum hist_type type, gimple *stmt, tree value)
 {
    histogram_value hist = (histogram_value) xcalloc (1, sizeof (*hist));
    hist->hvalue.value = value;
@@ -191,13 +141,13 @@ histogram_hash (const void *x)
 static int
 histogram_eq (const void *x, const void *y)
 {
-  return ((const_histogram_value) x)->hvalue.stmt == (const_gimple) y;
+  return ((const_histogram_value) x)->hvalue.stmt == (const gimple *) y;
 }
 
 /* Set histogram for STMT.  */
 
 static void
-set_histogram_value (struct function *fun, gimple stmt, histogram_value hist)
+set_histogram_value (struct function *fun, gimple *stmt, histogram_value hist)
 {
   void **loc;
   if (!hist && !VALUE_HISTOGRAMS (fun))
@@ -220,7 +170,7 @@ set_histogram_value (struct function *fun, gimple stmt, histogram_value hist)
 /* Get histogram list for STMT.  */
 
 histogram_value
-gimple_histogram_value (struct function *fun, gimple stmt)
+gimple_histogram_value (struct function *fun, gimple *stmt)
 {
   if (!VALUE_HISTOGRAMS (fun))
     return NULL;
@@ -231,7 +181,7 @@ gimple_histogram_value (struct function *fun, gimple stmt)
 /* Add histogram for STMT.  */
 
 void
-gimple_add_histogram_value (struct function *fun, gimple stmt,
+gimple_add_histogram_value (struct function *fun, gimple *stmt,
 			    histogram_value hist)
 {
   hist->hvalue.next = gimple_histogram_value (fun, stmt);
@@ -239,11 +189,10 @@ gimple_add_histogram_value (struct function *fun, gimple stmt,
   hist->fun = fun;
 }
 
-
 /* Remove histogram HIST from STMT's histogram list.  */
 
 void
-gimple_remove_histogram_value (struct function *fun, gimple stmt,
+gimple_remove_histogram_value (struct function *fun, gimple *stmt,
 			       histogram_value hist)
 {
   histogram_value hist2 = gimple_histogram_value (fun, stmt);
@@ -258,17 +207,15 @@ gimple_remove_histogram_value (struct function *fun, gimple stmt,
       hist2->hvalue.next = hist->hvalue.next;
     }
   free (hist->hvalue.counters);
-#ifdef ENABLE_CHECKING
-  memset (hist, 0xab, sizeof (*hist));
-#endif
+  if (flag_checking)
+    memset (hist, 0xab, sizeof (*hist));
   free (hist);
 }
 
-
 /* Lookup histogram of type TYPE in the STMT.  */
 
 histogram_value
-gimple_histogram_value_of_type (struct function *fun, gimple stmt,
+gimple_histogram_value_of_type (struct function *fun, gimple *stmt,
 				enum hist_type type)
 {
   histogram_value hist;
@@ -296,10 +243,10 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
 	   unsigned int i;
 	   fprintf (dump_file, " [");
            for (i = 0; i < hist->hdata.intvl.steps; i++)
-	     fprintf (dump_file, " %d:%"PRId64,
+	     fprintf (dump_file, " %d:%" PRId64,
 		      hist->hdata.intvl.int_start + i,
 		      (int64_t) hist->hvalue.counters[i]);
-	   fprintf (dump_file, " ] outside range:%"PRId64,
+	   fprintf (dump_file, " ] outside range:%" PRId64,
 		    (int64_t) hist->hvalue.counters[i]);
 	}
       fprintf (dump_file, ".\n");
@@ -309,10 +256,10 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
       fprintf (dump_file, "Pow2 counter ");
       if (hist->hvalue.counters)
 	{
-	   fprintf (dump_file, "pow2:%"PRId64
-		    " nonpow2:%"PRId64,
-		    (int64_t) hist->hvalue.counters[0],
-		    (int64_t) hist->hvalue.counters[1]);
+	   fprintf (dump_file, "pow2:%" PRId64
+		    " nonpow2:%" PRId64,
+		    (int64_t) hist->hvalue.counters[1],
+		    (int64_t) hist->hvalue.counters[0]);
 	}
       fprintf (dump_file, ".\n");
       break;
@@ -321,9 +268,9 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
       fprintf (dump_file, "Single value ");
       if (hist->hvalue.counters)
 	{
-	   fprintf (dump_file, "value:%"PRId64
-		    " match:%"PRId64
-		    " wrong:%"PRId64,
+	   fprintf (dump_file, "value:%" PRId64
+		    " match:%" PRId64
+		    " wrong:%" PRId64,
 		    (int64_t) hist->hvalue.counters[0],
 		    (int64_t) hist->hvalue.counters[1],
 		    (int64_t) hist->hvalue.counters[2]);
@@ -335,8 +282,8 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
       fprintf (dump_file, "Average value ");
       if (hist->hvalue.counters)
 	{
-	   fprintf (dump_file, "sum:%"PRId64
-		    " times:%"PRId64,
+	   fprintf (dump_file, "sum:%" PRId64
+		    " times:%" PRId64,
 		    (int64_t) hist->hvalue.counters[0],
 		    (int64_t) hist->hvalue.counters[1]);
 	}
@@ -347,32 +294,19 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
       fprintf (dump_file, "IOR value ");
       if (hist->hvalue.counters)
 	{
-	   fprintf (dump_file, "ior:%"PRId64,
+	   fprintf (dump_file, "ior:%" PRId64,
 		    (int64_t) hist->hvalue.counters[0]);
 	}
       fprintf (dump_file, ".\n");
       break;
 
-    case HIST_TYPE_CONST_DELTA:
-      fprintf (dump_file, "Constant delta ");
-      if (hist->hvalue.counters)
-	{
-	   fprintf (dump_file, "value:%"PRId64
-		    " match:%"PRId64
-		    " wrong:%"PRId64,
-		    (int64_t) hist->hvalue.counters[0],
-		    (int64_t) hist->hvalue.counters[1],
-		    (int64_t) hist->hvalue.counters[2]);
-	}
-      fprintf (dump_file, ".\n");
-      break;
     case HIST_TYPE_INDIR_CALL:
       fprintf (dump_file, "Indirect call ");
       if (hist->hvalue.counters)
 	{
-	   fprintf (dump_file, "value:%"PRId64
-		    " match:%"PRId64
-		    " all:%"PRId64,
+	   fprintf (dump_file, "value:%" PRId64
+		    " match:%" PRId64
+		    " all:%" PRId64,
 		    (int64_t) hist->hvalue.counters[0],
 		    (int64_t) hist->hvalue.counters[1],
 		    (int64_t) hist->hvalue.counters[2]);
@@ -383,7 +317,7 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
       fprintf (dump_file, "Time profile ");
       if (hist->hvalue.counters)
       {
-        fprintf (dump_file, "time:%"PRId64,
+        fprintf (dump_file, "time:%" PRId64,
                  (int64_t) hist->hvalue.counters[0]);
       }
       fprintf (dump_file, ".\n");
@@ -394,10 +328,10 @@ dump_histogram_value (FILE *dump_file, histogram_value hist)
 	{
            int i;
 
-           fprintf (dump_file, "accu:%"PRId64, hist->hvalue.counters[0]);
+           fprintf (dump_file, "accu:%" PRId64, hist->hvalue.counters[0]);
            for (i = 1; i < (GCOV_ICALL_TOPN_VAL << 2); i += 2)
              {
-               fprintf (dump_file, " target:%"PRId64 " value:%"PRId64,
+               fprintf (dump_file, " target:%" PRId64 " value:%" PRId64,
                        (int64_t) hist->hvalue.counters[i],
                        (int64_t) hist->hvalue.counters[i+1]);
              }
@@ -445,10 +379,11 @@ stream_out_histogram_value (struct output_block *ob, histogram_value hist)
   if (hist->hvalue.next)
     stream_out_histogram_value (ob, hist->hvalue.next);
 }
+
 /* Dump information about HIST to DUMP_FILE.  */
 
 void
-stream_in_histogram_value (struct lto_input_block *ib, gimple stmt)
+stream_in_histogram_value (struct lto_input_block *ib, gimple *stmt)
 {
   enum hist_type type;
   unsigned int ncounters = 0;
@@ -482,10 +417,6 @@ stream_in_histogram_value (struct lto_input_block *ib, gimple stmt)
 	  ncounters = 3;
 	  break;
 
-	case HIST_TYPE_CONST_DELTA:
-	  ncounters = 4;
-	  break;
-
 	case HIST_TYPE_IOR:
         case HIST_TYPE_TIME_PROFILE:
 	  ncounters = 1;
@@ -514,7 +445,7 @@ stream_in_histogram_value (struct lto_input_block *ib, gimple stmt)
 /* Dump all histograms attached to STMT to DUMP_FILE.  */
 
 void
-dump_histograms_for_stmt (struct function *fun, FILE *dump_file, gimple stmt)
+dump_histograms_for_stmt (struct function *fun, FILE *dump_file, gimple *stmt)
 {
   histogram_value hist;
   for (hist = gimple_histogram_value (fun, stmt); hist; hist = hist->hvalue.next)
@@ -524,7 +455,7 @@ dump_histograms_for_stmt (struct function *fun, FILE *dump_file, gimple stmt)
 /* Remove all histograms associated with STMT.  */
 
 void
-gimple_remove_stmt_histograms (struct function *fun, gimple stmt)
+gimple_remove_stmt_histograms (struct function *fun, gimple *stmt)
 {
   histogram_value val;
   while ((val = gimple_histogram_value (fun, stmt)) != NULL)
@@ -534,8 +465,8 @@ gimple_remove_stmt_histograms (struct function *fun, gimple stmt)
 /* Duplicate all histograms associates with OSTMT to STMT.  */
 
 void
-gimple_duplicate_stmt_histograms (struct function *fun, gimple stmt,
-				  struct function *ofun, gimple ostmt)
+gimple_duplicate_stmt_histograms (struct function *fun, gimple *stmt,
+				  struct function *ofun, gimple *ostmt)
 {
   histogram_value val;
   for (val = gimple_histogram_value (ofun, ostmt); val != NULL; val = val->hvalue.next)
@@ -549,11 +480,10 @@ gimple_duplicate_stmt_histograms (struct function *fun, gimple stmt,
     }
 }
 
-
 /* Move all histograms associated with OSTMT to STMT.  */
 
 void
-gimple_move_stmt_histograms (struct function *fun, gimple stmt, gimple ostmt)
+gimple_move_stmt_histograms (struct function *fun, gimple *stmt, gimple *ostmt)
 {
   histogram_value val = gimple_histogram_value (fun, ostmt);
   if (val)
@@ -590,7 +520,6 @@ visit_hist (void **slot, void *data)
   return 1;
 }
 
-
 /* Verify sanity of the histograms.  */
 
 DEBUG_FUNCTION void
@@ -605,7 +534,7 @@ verify_histograms (void)
   FOR_EACH_BB_FN (bb, cfun)
     for (gsi = gsi_start_bb (bb); !gsi_end_p (gsi); gsi_next (&gsi))
       {
-	gimple stmt = gsi_stmt (gsi);
+	gimple *stmt = gsi_stmt (gsi);
 
 	for (hist = gimple_histogram_value (cfun, stmt); hist;
 	     hist = hist->hvalue.next)
@@ -635,32 +564,30 @@ free_hist (void **slot, void *data ATTRIBUTE_UNUSED)
 {
   histogram_value hist = *(histogram_value *) slot;
   free (hist->hvalue.counters);
-#ifdef ENABLE_CHECKING
-  memset (hist, 0xab, sizeof (*hist));
-#endif
+  if (flag_checking)
+    memset (hist, 0xab, sizeof (*hist));
   free (hist);
   return 1;
 }
 
 void
-free_histograms (void)
+free_histograms (struct function *fn)
 {
-  if (VALUE_HISTOGRAMS (cfun))
+  if (VALUE_HISTOGRAMS (fn))
     {
-      htab_traverse (VALUE_HISTOGRAMS (cfun), free_hist, NULL);
-      htab_delete (VALUE_HISTOGRAMS (cfun));
-      VALUE_HISTOGRAMS (cfun) = NULL;
+      htab_traverse (VALUE_HISTOGRAMS (fn), free_hist, NULL);
+      htab_delete (VALUE_HISTOGRAMS (fn));
+      VALUE_HISTOGRAMS (fn) = NULL;
     }
 }
 
-
 /* The overall number of invocations of the counter should match
    execution count of basic block.  Report it as error rather than
    internal error as it might mean that user has misused the profile
    somehow.  */
 
 static bool
-check_counter (gimple stmt, const char * name,
+check_counter (gimple *stmt, const char * name,
 	       gcov_type *count, gcov_type *all, gcov_type bb_count)
 {
   if (*all != bb_count || *count > *all)
@@ -697,7 +624,6 @@ check_counter (gimple stmt, const char * name,
   return false;
 }
 
-
 /* GIMPLE based transformations. */
 
 bool
@@ -707,11 +633,16 @@ gimple_value_profile_transformations (void)
   gimple_stmt_iterator gsi;
   bool changed = false;
 
+  /* Autofdo does its own transformations for indirect calls,
+     and otherwise does not support value profiling.  */
+  if (flag_auto_profile)
+    return false;
+
   FOR_EACH_BB_FN (bb, cfun)
     {
       for (gsi = gsi_start_bb (bb); !gsi_end_p (gsi); gsi_next (&gsi))
 	{
-	  gimple stmt = gsi_stmt (gsi);
+	  gimple *stmt = gsi_stmt (gsi);
 	  histogram_value th = gimple_histogram_value (cfun, stmt);
 	  if (!th)
 	    continue;
@@ -756,7 +687,6 @@ gimple_value_profile_transformations (void)
   return changed;
 }
 
-
 /* Generate code for transformation 1 (with parent gimple assignment
    STMT and probability of taking the optimal path PROB, which is
    equivalent to COUNT/ALL within roundoff error).  This generates the
@@ -770,7 +700,7 @@ gimple_divmod_fixed_value (gassign *stmt, tree value, int prob,
   gassign *stmt1, *stmt2;
   gcond *stmt3;
   tree tmp0, tmp1, tmp2;
-  gimple bb1end, bb2end, bb3end;
+  gimple *bb1end, *bb2end, *bb3end;
   basic_block bb, bb2, bb3, bb4;
   tree optype, op1, op2;
   edge e12, e13, e23, e24, e34;
@@ -839,7 +769,6 @@ gimple_divmod_fixed_value (gassign *stmt, tree value, int prob,
   return tmp2;
 }
 
-
 /* Do transform 1) on INSN if applicable.  */
 
 static bool
@@ -925,13 +854,14 @@ gimple_divmod_fixed_value_transform (gimple_stmt_iterator *si)
    probability of taking the optimal path PROB, which is equivalent to COUNT/ALL
    within roundoff error).  This generates the result into a temp and returns
    the temp; it does not replace or alter the original STMT.  */
+
 static tree
 gimple_mod_pow2 (gassign *stmt, int prob, gcov_type count, gcov_type all)
 {
   gassign *stmt1, *stmt2, *stmt3;
   gcond *stmt4;
   tree tmp2, tmp3;
-  gimple bb1end, bb2end, bb3end;
+  gimple *bb1end, *bb2end, *bb3end;
   basic_block bb, bb2, bb3, bb4;
   tree optype, op1, op2;
   edge e12, e13, e23, e24, e34;
@@ -1005,6 +935,7 @@ gimple_mod_pow2 (gassign *stmt, int prob, gcov_type count, gcov_type all)
 }
 
 /* Do transform 2) on INSN if applicable.  */
+
 static bool
 gimple_mod_pow2_value_transform (gimple_stmt_iterator *si)
 {
@@ -1083,10 +1014,10 @@ gimple_mod_subtract (gassign *stmt, int prob1, int prob2, int ncounts,
 		     gcov_type count1, gcov_type count2, gcov_type all)
 {
   gassign *stmt1;
-  gimple stmt2;
+  gimple *stmt2;
   gcond *stmt3;
   tree tmp1;
-  gimple bb1end, bb2end = NULL, bb3end;
+  gimple *bb1end, *bb2end = NULL, *bb3end;
   basic_block bb, bb2, bb3, bb4;
   tree optype, op1, op2;
   edge e12, e23 = 0, e24, e34, e14;
@@ -1174,7 +1105,6 @@ gimple_mod_subtract (gassign *stmt, int prob1, int prob2, int ncounts,
   return result;
 }
 
-
 /* Do transforms 3) and 4) on the statement pointed-to by SI if applicable.  */
 
 static bool
@@ -1188,7 +1118,6 @@ gimple_mod_subtract_transform (gimple_stmt_iterator *si)
   unsigned int i, steps;
   gcov_type count1, count2;
   gassign *stmt;
-
   stmt = dyn_cast <gassign *> (gsi_stmt (*si));
   if (!stmt)
     return false;
@@ -1271,22 +1200,9 @@ gimple_mod_subtract_transform (gimple_stmt_iterator *si)
   return true;
 }
 
-struct profile_id_traits : default_hashmap_traits
-{
-  template<typename T>
-  static bool
-  is_deleted (T &e)
-    {
-      return e.m_key == UINT_MAX;
-    }
+typedef int_hash <unsigned int, 0, UINT_MAX> profile_id_hash;
 
-  template<typename T> static bool is_empty (T &e) { return e.m_key == 0; }
-  template<typename T> static void mark_deleted (T &e) { e.m_key = UINT_MAX; }
-  template<typename T> static void mark_empty (T &e) { e.m_key = 0; }
-};
-
-static hash_map<unsigned int, cgraph_node *, profile_id_traits> *
-cgraph_node_map = 0;
+static hash_map<profile_id_hash, cgraph_node *> *cgraph_node_map = 0;
 
 /* Returns true if node graph is initialized. This
    is used to test if profile_id has been created
@@ -1306,8 +1222,7 @@ void
 init_node_map (bool local)
 {
   struct cgraph_node *n;
-  cgraph_node_map
-    = new hash_map<unsigned int, cgraph_node *, profile_id_traits>;
+  cgraph_node_map = new hash_map<profile_id_hash, cgraph_node *>;
 
   FOR_EACH_DEFINED_FUNCTION (n)
     if (n->has_gimple_body_p ())
@@ -1435,21 +1350,26 @@ gimple_ic (gcall *icall_stmt, struct cgraph_node *direct_call,
   load_stmt = gimple_build_assign (tmp0, tmp);
   gsi_insert_before (&gsi, load_stmt, GSI_SAME_STMT);
 
-  tmp = fold_convert (optype, build_addr (direct_call->decl,
-					  current_function_decl));
+  tmp = fold_convert (optype, build_addr (direct_call->decl));
   load_stmt = gimple_build_assign (tmp1, tmp);
   gsi_insert_before (&gsi, load_stmt, GSI_SAME_STMT);
 
   cond_stmt = gimple_build_cond (EQ_EXPR, tmp1, tmp0, NULL_TREE, NULL_TREE);
   gsi_insert_before (&gsi, cond_stmt, GSI_SAME_STMT);
 
+  if (TREE_CODE (gimple_vdef (icall_stmt)) == SSA_NAME)
+    {
+      unlink_stmt_vdef (icall_stmt);
+      release_ssa_name (gimple_vdef (icall_stmt));
+    }
   gimple_set_vdef (icall_stmt, NULL_TREE);
   gimple_set_vuse (icall_stmt, NULL_TREE);
   update_stmt (icall_stmt);
   dcall_stmt = as_a <gcall *> (gimple_copy (icall_stmt));
   gimple_call_set_fndecl (dcall_stmt, direct_call->decl);
   dflags = flags_from_decl_or_type (direct_call->decl);
-  if ((dflags & ECF_NORETURN) != 0)
+  if ((dflags & ECF_NORETURN) != 0
+      && should_remove_lhs_p (gimple_call_lhs (dcall_stmt)))
     gimple_call_set_lhs (dcall_stmt, NULL_TREE);
   gsi_insert_before (&gsi, dcall_stmt, GSI_SAME_STMT);
 
@@ -1529,8 +1449,13 @@ gimple_ic (gcall *icall_stmt, struct cgraph_node *direct_call,
 	{
 	  if (gimple_call_lhs (iretbnd_stmt))
 	    {
-	      gimple copy;
+	      gimple *copy;
 
+	      if (TREE_CODE (gimple_vdef (iretbnd_stmt)) == SSA_NAME)
+		{
+	          unlink_stmt_vdef (iretbnd_stmt);
+	          release_ssa_name (gimple_vdef (iretbnd_stmt));
+		}
 	      gimple_set_vdef (iretbnd_stmt, NULL_TREE);
 	      gimple_set_vuse (iretbnd_stmt, NULL_TREE);
 	      update_stmt (iretbnd_stmt);
@@ -1676,22 +1601,22 @@ gimple_ic_transform (gimple_stmt_iterator *gsi)
       print_generic_expr (dump_file, direct_call->decl, TDF_SLIM);
       fprintf (dump_file, " transformation on insn postponned to ipa-profile");
       print_gimple_stmt (dump_file, stmt, 0, TDF_SLIM);
-      fprintf (dump_file, "hist->count %"PRId64
-	       " hist->all %"PRId64"\n", count, all);
+      fprintf (dump_file, "hist->count %" PRId64
+	       " hist->all %" PRId64"\n", count, all);
     }
 
   return true;
 }
 
-/* Return true if the stringop CALL with FNDECL shall be profiled.
-   SIZE_ARG be set to the argument index for the size of the string
-   operation.
-*/
-static bool
-interesting_stringop_to_profile_p (tree fndecl, gcall *call, int *size_arg)
-{
-  enum built_in_function fcode = DECL_FUNCTION_CODE (fndecl);
+/* Return true if the stringop CALL shall be profiled.  SIZE_ARG be
+   set to the argument index for the size of the string operation.  */
 
+static bool
+interesting_stringop_to_profile_p (gcall *call, int *size_arg)
+{
+  enum built_in_function fcode;
+
+  fcode = DECL_FUNCTION_CODE (gimple_call_fndecl (call));
   if (fcode != BUILT_IN_MEMCPY && fcode != BUILT_IN_MEMPCPY
       && fcode != BUILT_IN_MEMSET && fcode != BUILT_IN_BZERO)
     return false;
@@ -1716,7 +1641,7 @@ interesting_stringop_to_profile_p (tree fndecl, gcall *call, int *size_arg)
     }
 }
 
-/* Convert   stringop (..., vcall_size)
+/* Convert stringop (..., vcall_size)
    into
    if (vcall_size == icall_size)
      stringop (..., icall_size);
@@ -1735,11 +1660,9 @@ gimple_stringop_fixed_value (gcall *vcall_stmt, tree icall_size, int prob,
   basic_block cond_bb, icall_bb, vcall_bb, join_bb;
   edge e_ci, e_cv, e_iv, e_ij, e_vj;
   gimple_stmt_iterator gsi;
-  tree fndecl;
   int size_arg;
 
-  fndecl = gimple_call_fndecl (vcall_stmt);
-  if (!interesting_stringop_to_profile_p (fndecl, vcall_stmt, &size_arg))
+  if (!interesting_stringop_to_profile_p (vcall_stmt, &size_arg))
     gcc_unreachable ();
 
   cond_bb = gimple_bb (vcall_stmt);
@@ -1759,11 +1682,17 @@ gimple_stringop_fixed_value (gcall *vcall_stmt, tree icall_size, int prob,
   cond_stmt = gimple_build_cond (EQ_EXPR, tmp1, tmp0, NULL_TREE, NULL_TREE);
   gsi_insert_before (&gsi, cond_stmt, GSI_SAME_STMT);
 
+  if (TREE_CODE (gimple_vdef (vcall_stmt)) == SSA_NAME)
+    {
+      unlink_stmt_vdef (vcall_stmt);
+      release_ssa_name (gimple_vdef (vcall_stmt));
+    }
   gimple_set_vdef (vcall_stmt, NULL);
   gimple_set_vuse (vcall_stmt, NULL);
   update_stmt (vcall_stmt);
   icall_stmt = as_a <gcall *> (gimple_copy (vcall_stmt));
-  gimple_call_set_arg (icall_stmt, size_arg, icall_size);
+  gimple_call_set_arg (icall_stmt, size_arg,
+		       fold_convert (optype, icall_size));
   gsi_insert_before (&gsi, icall_stmt, GSI_SAME_STMT);
 
   /* Fix CFG. */
@@ -1818,11 +1747,11 @@ gimple_stringop_fixed_value (gcall *vcall_stmt, tree icall_size, int prob,
 
 /* Find values inside STMT for that we want to measure histograms for
    division/modulo optimization.  */
+
 static bool
 gimple_stringops_transform (gimple_stmt_iterator *gsi)
 {
   gcall *stmt;
-  tree fndecl;
   tree blck_size;
   enum built_in_function fcode;
   histogram_value histogram;
@@ -1836,11 +1765,11 @@ gimple_stringops_transform (gimple_stmt_iterator *gsi)
   stmt = dyn_cast <gcall *> (gsi_stmt (*gsi));
   if (!stmt)
     return false;
-  fndecl = gimple_call_fndecl (stmt);
-  if (!fndecl)
+
+  if (!gimple_call_builtin_p (gsi_stmt (*gsi), BUILT_IN_NORMAL))
     return false;
-  fcode = DECL_FUNCTION_CODE (fndecl);
-  if (!interesting_stringop_to_profile_p (fndecl, stmt, &size_arg))
+
+  if (!interesting_stringop_to_profile_p (stmt, &size_arg))
     return false;
 
   blck_size = gimple_call_arg (stmt, size_arg);
@@ -1850,10 +1779,12 @@ gimple_stringops_transform (gimple_stmt_iterator *gsi)
   histogram = gimple_histogram_value_of_type (cfun, stmt, HIST_TYPE_SINGLE_VALUE);
   if (!histogram)
     return false;
+
   val = histogram->hvalue.counters[0];
   count = histogram->hvalue.counters[1];
   all = histogram->hvalue.counters[2];
   gimple_remove_histogram_value (cfun, stmt, histogram);
+
   /* We require that count is at least half of all; this means
      that for the transformation to fire the value must be constant
      at least 80% of time.  */
@@ -1865,8 +1796,10 @@ gimple_stringops_transform (gimple_stmt_iterator *gsi)
     prob = GCOV_COMPUTE_SCALE (count, all);
   else
     prob = 0;
+
   dest = gimple_call_arg (stmt, 0);
   dest_align = get_pointer_alignment (dest);
+  fcode = DECL_FUNCTION_CODE (gimple_call_fndecl (stmt));
   switch (fcode)
     {
     case BUILT_IN_MEMCPY:
@@ -1891,6 +1824,7 @@ gimple_stringops_transform (gimple_stmt_iterator *gsi)
     default:
       gcc_unreachable ();
     }
+
   if (sizeof (gcov_type) == sizeof (HOST_WIDE_INT))
     tree_val = build_int_cst (get_gcov_type (), val);
   else
@@ -1909,17 +1843,19 @@ gimple_stringops_transform (gimple_stmt_iterator *gsi)
 	       (int)val);
       print_gimple_stmt (dump_file, stmt, 0, TDF_SLIM);
     }
+
   gimple_stringop_fixed_value (stmt, tree_val, prob, count, all);
 
   return true;
 }
 
 void
-stringop_block_profile (gimple stmt, unsigned int *expected_align,
+stringop_block_profile (gimple *stmt, unsigned int *expected_align,
 			HOST_WIDE_INT *expected_size)
 {
   histogram_value histogram;
   histogram = gimple_histogram_value_of_type (cfun, stmt, HIST_TYPE_AVERAGE);
+
   if (!histogram)
     *expected_size = -1;
   else if (!histogram->hvalue.counters[1])
@@ -1940,7 +1876,9 @@ stringop_block_profile (gimple stmt, unsigned int *expected_align,
       *expected_size = size;
       gimple_remove_histogram_value (cfun, stmt, histogram);
     }
+
   histogram = gimple_histogram_value_of_type (cfun, stmt, HIST_TYPE_IOR);
+
   if (!histogram)
     *expected_align = 0;
   else if (!histogram->hvalue.counters[0])
@@ -1951,12 +1889,12 @@ stringop_block_profile (gimple stmt, unsigned int *expected_align,
   else
     {
       gcov_type count;
-      int alignment;
+      unsigned int alignment;
 
       count = histogram->hvalue.counters[0];
       alignment = 1;
       while (!(count & alignment)
-	     && (alignment * 2 * BITS_PER_UNIT))
+	     && (alignment <= UINT_MAX / 2 / BITS_PER_UNIT))
 	alignment <<= 1;
       *expected_align = alignment * BITS_PER_UNIT;
       gimple_remove_histogram_value (cfun, stmt, histogram);
@@ -1966,8 +1904,9 @@ stringop_block_profile (gimple stmt, unsigned int *expected_align,
 
 /* Find values inside STMT for that we want to measure histograms for
    division/modulo optimization.  */
+
 static void
-gimple_divmod_values_to_profile (gimple stmt, histogram_values *values)
+gimple_divmod_values_to_profile (gimple *stmt, histogram_values *values)
 {
   tree lhs, divisor, op0, type;
   histogram_value hist;
@@ -1999,7 +1938,8 @@ gimple_divmod_values_to_profile (gimple stmt, histogram_values *values)
       /* For mod, check whether it is not often a noop (or replaceable by
 	 a few subtractions).  */
       if (gimple_assign_rhs_code (stmt) == TRUNC_MOD_EXPR
-	  && TYPE_UNSIGNED (type))
+	  && TYPE_UNSIGNED (type)
+	  && TREE_CODE (divisor) == SSA_NAME)
 	{
           tree val;
           /* Check for a special case where the divisor is power of 2.  */
@@ -2025,7 +1965,7 @@ gimple_divmod_values_to_profile (gimple stmt, histogram_values *values)
    indirect/virtual call optimization. */
 
 static void
-gimple_indirect_call_to_profile (gimple stmt, histogram_values *values)
+gimple_indirect_call_to_profile (gimple *stmt, histogram_values *values)
 {
   tree callee;
 
@@ -2050,11 +1990,11 @@ gimple_indirect_call_to_profile (gimple stmt, histogram_values *values)
 
 /* Find values inside STMT for that we want to measure histograms for
    string operations.  */
+
 static void
-gimple_stringops_values_to_profile (gimple gs, histogram_values *values)
+gimple_stringops_values_to_profile (gimple *gs, histogram_values *values)
 {
   gcall *stmt;
-  tree fndecl;
   tree blck_size;
   tree dest;
   int size_arg;
@@ -2062,11 +2002,11 @@ gimple_stringops_values_to_profile (gimple gs, histogram_values *values)
   stmt = dyn_cast <gcall *> (gs);
   if (!stmt)
     return;
-  fndecl = gimple_call_fndecl (stmt);
-  if (!fndecl)
+
+  if (!gimple_call_builtin_p (gs, BUILT_IN_NORMAL))
     return;
 
-  if (!interesting_stringop_to_profile_p (fndecl, stmt, &size_arg))
+  if (!interesting_stringop_to_profile_p (stmt, &size_arg))
     return;
 
   dest = gimple_call_arg (stmt, 0);
@@ -2080,6 +2020,7 @@ gimple_stringops_values_to_profile (gimple gs, histogram_values *values)
       values->safe_push (gimple_alloc_histogram_value (cfun, HIST_TYPE_AVERAGE,
 						       stmt, blck_size));
     }
+
   if (TREE_CODE (blck_size) != INTEGER_CST)
     values->safe_push (gimple_alloc_histogram_value (cfun, HIST_TYPE_IOR,
 						     stmt, dest));
@@ -2089,7 +2030,7 @@ gimple_stringops_values_to_profile (gimple gs, histogram_values *values)
    them to list VALUES.  */
 
 static void
-gimple_values_to_profile (gimple stmt, histogram_values *values)
+gimple_values_to_profile (gimple *stmt, histogram_values *values)
 {
   gimple_divmod_values_to_profile (stmt, values);
   gimple_stringops_values_to_profile (stmt, values);
@@ -2127,10 +2068,6 @@ gimple_find_values_to_profile (histogram_values *values)
 	  hist->n_counters = 3;
 	  break;
 
-	case HIST_TYPE_CONST_DELTA:
-	  hist->n_counters = 4;
-	  break;
-
  	case HIST_TYPE_INDIR_CALL:
  	  hist->n_counters = 3;
 	  break;
