@@ -1,6 +1,6 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 
-// Copyright (C) 2007-2015 Free Software Foundation, Inc.
+// Copyright (C) 2007-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,7 +26,6 @@
 // Make sure each invocation of what() doesn't grow the message.
 void test01()
 {
-  bool test __attribute__((unused)) = true;
   std::string s("after nine thirty, this request cannot be met");
 
   std::system_error obj =
