@@ -1,8 +1,8 @@
-// { dg-require-namedlocale "de_DE@euro" }
+// { dg-require-namedlocale "de_DE.ISO8859-15" }
 
 // 2003-10-24  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2003-2015 Free Software Foundation, Inc.
+// Copyright (C) 2003-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -30,11 +30,9 @@ void test01()
 {
   using namespace std;
   typedef istreambuf_iterator<wchar_t> iterator_type;
-
-  bool test __attribute__((unused)) = true;
   
   // basic construction
-  locale loc_de = locale("de_DE@euro");
+  locale loc_de = locale(ISO_8859(15,de_DE));
 
   iterator_type end;
   wistringstream iss;
