@@ -1,6 +1,6 @@
 // 2007-04-27  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation
+// Copyright (C) 2007-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,7 +18,7 @@
 // <http://www.gnu.org/licenses/>.
 
 // { dg-do compile }
-// { dg-error "no matching" "" { target *-*-* } 1508 }
+// { dg-prune-output "no matching function .*_M_fill_initialize" }
 
 #include <list>
 #include <utility>
@@ -26,5 +26,5 @@
 void f()
 {
   typedef std::list<std::list<std::pair<char, char> > > list_type;
-  list_type l('a', 'b');
+  list_type l('a', 'b');	// { dg-error "here|no match" }
 }
