@@ -40,35 +40,40 @@ void test01()
   basic_filebuf<pod_uchar> fb;
   fb.pubimbue(loc);
 
-  fb.open(name, ios_base::out | ios_base::trunc);
-  fb.sputc(pod_uchar::from<char>('b'));
+  fb.open(name, ios_base::out);
   fb.sputc(pod_uchar::from<char>(0xff));
+  fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>(0));
+  fb.sputc(pod_uchar::from<char>('a'));
+  fb.sputc(pod_uchar::from<char>('a'));
+  fb.sputc(pod_uchar::from<char>('a'));
   fb.sputc(pod_uchar::from<char>('a'));
-  fb.sputc(pod_uchar::from<char>(0xfc));
-  fb.sputc(pod_uchar::from<char>(0));
-  fb.sputc(pod_uchar::from<char>(0));
-
   fb.close();
+
   fb.open(name, ios_base::in);
-
+  fb.sbumpc();
   fb.sbumpc();
   fb.sbumpc();
 
-  // Check that seekoff returns the correct state
-  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::cur);
+  // Check that seekoff resets the state when seeking to beginning.
+  traits_type::pos_type pos = fb.pubseekoff(0, ios_base::beg);
+  VERIFY( pos != traits_type::pos_type(traits_type::off_type(-1)) );
 
   traits_type::int_type c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
-			  pod_uchar::from<char>('a')) );
+			  pod_uchar::from<char>(0xff)) );
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
+  fb.sbumpc();
   fb.sbumpc();
-
-  fb.pubseekpos(pos);
-
   c = fb.sbumpc();
   VERIFY( c != traits_type::eof() );
   VERIFY( traits_type::eq(traits_type::to_char_type(c),
 			  pod_uchar::from<char>('a')) );
+  c = fb.sbumpc();
+  VERIFY( c == traits_type::eof() );
 
   fb.close();
 }
