@@ -15,12 +15,8 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-
 // 27.6.1.1.2 class basic_istream::sentry
 
-// The library throws the new definition of std::ios::failure
-// { dg-options "-D_GLIBCXX_USE_CXX11_ABI=1" }
-
 #include <sstream>
 #include <testsuite_hooks.h>
 
@@ -29,16 +25,16 @@ int main()
   using namespace std;
   istringstream stream;
   stream.exceptions(ios_base::eofbit);
-  
+
   try
     {
       istream::sentry sentry(stream, false);
       VERIFY( false );
     }
-  catch (ios_base::failure&)
+  catch (std::ios_base::failure&)
     {
       VERIFY( stream.rdstate() == (ios_base::eofbit | ios_base::failbit) );
     }
-  
+
   return 0;
 }
